#include "patchlevel.h"
#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#ifdef NO_XPM

#include "writs1.xbm"
#include "writs2.xbm"
#include "writsres.xbm"
#include "writsok.xbm"
#define SETUPPIX(data, id) pix[id] = None; pixdata[id] = data##_bits; pixwidth[id] = data##_width; pixheight[id] = data##_height; pixbg[id] = data##_bg

#else

#include "writs1.xpm"
#include "writs2.xpm"
#include "writsres.xpm"
#include "writsok.xpm"
#ifndef XPM_HEADER
#define XPM_HEADER <xpm.h>
#endif
#include XPM_HEADER
#define SETUPPIX(data, id) pix[id] = None; pixdata[id] = data##_xpm

#endif

#include <X11/Xos.h>
#ifndef FD_SET
#include <sys/select.h>
#endif

#ifndef DEFAULT_TYPETIME
#define DEFAULT_TYPETIME 55
#endif
#ifndef DEFAULT_BREAKTIME
#define DEFAULT_BREAKTIME 5
#endif
#ifndef FLASHSECONDS
#define FLASHSECONDS 2
#endif

Display *display;
Window window;
GC gc;
char *displayname = 0;
int typetime = DEFAULT_TYPETIME;
int breaktime = DEFAULT_BREAKTIME;
int flashseconds = FLASHSECONDS;

#define PixBreak1   0
#define PixBreak2   1
#define PixResting  2
#define PixOK       3

Pixmap pix[4];

#ifdef NO_XPM

unsigned char *pixdata[4];
int pixwidth[4];
int pixheight[4];
char *pixbg[4];

#else

char **pixdata[4];

#endif

#define WindowWidth 300
#define WindowHeight 300

#ifdef NO_XPM

Pixmap
make_pixmap(int num)
{
  Pixmap p;
  XColor bg;
  XWindowAttributes attr;
  
  XGetWindowAttributes(display, window, &attr);
  if (!XParseColor(display, attr.colormap, pixbg[num], &bg) ||
      !XAllocColor(display, attr.colormap, &bg))
    bg.pixel = WhitePixelOfScreen(attr.screen);
  
  p = XCreatePixmapFromBitmapData(display, window, pixdata[num],
				  pixwidth[num], pixheight[num],
				  BlackPixelOfScreen(attr.screen),
				  bg.pixel,
				  attr.depth);
  return p;
}

#else

Pixmap
make_pixmap(int num)
{
  Pixmap p;
  static XpmAttributes *xpma = 0;
  if (!xpma) {
    xpma = (XpmAttributes *)malloc(XpmAttributesSize());
    xpma->closeness = 200;
    xpma->valuemask = XpmCloseness;
  }
  XpmCreatePixmapFromData(display, window, pixdata[num], &p, 0, xpma);
  return p;
}

#endif


void
error(char *x)
{
  fprintf(stderr, "xwrits: %s.\n", x);
  exit(1);
}


void
setpicture(int num)
{
  if (pix[num] == None) {
    pix[num] = make_pixmap(num);
    if (!pix[num]) error("couldn't make a pixmap");
  }
  XSetWindowBackgroundPixmap(display, window, pix[num]);
  XClearWindow(display, window);
  XFlush(display);
}


int
argcmp(char *argname, char *arggiven, int minchars)
{
  while (*argname && *arggiven) {
    if (*argname++ != *arggiven++) return 0;
    minchars--;
  }
  return (minchars <= 0);
}


#define argshift(n) do { argc -= n; argv += n; } while (0)


int
main(int argc, char *argv[])
{
  XEvent event;
  int cenx, ceny;
  int xsocket;
  
  argshift(1);
  
  while (argc > 0) {
    if (argcmp("-display", argv[0], 2)) {
      if (argc < 2) error("no argument given to -display");
      displayname = argv[1];
      argshift(2);
    } else if (argcmp("-typetime", argv[0], 2)) {
      if (argc < 2) error("no argument given to -typetime");
      typetime = strtol(argv[1], 0, 0);
      if (typetime < 0) typetime = 0;
      argshift(2);
    } else if (argcmp("-breaktime", argv[0], 2)) {
      if (argc < 2) error("no argument given to -breaktime");
      breaktime = strtol(argv[1], 0, 0);
      if (breaktime < 0) breaktime = 0;
      argshift(2);
    } else error("unrecognized argument");
  }
  
  display = XOpenDisplay(displayname);
  if (!display) error("cannot open display");
  xsocket = ConnectionNumber(display);
  
  cenx = (DisplayWidth(display, DefaultScreen(display)) - WindowWidth) / 2;
  ceny = (DisplayHeight(display, DefaultScreen(display)) - WindowHeight) / 2;
  window = XCreateSimpleWindow
    (display, DefaultRootWindow(display),
     cenx, ceny, WindowWidth, WindowHeight, 0, 1, 0);
  {
    XClassHint classh;
    XSizeHints *xsh;
    char *woog[2];
    XTextProperty windowname, iconname;
    woog[0] = "xwrits";
    woog[1] = NULL;
    XStringListToTextProperty(woog, 1, &windowname);
    XStringListToTextProperty(woog, 1, &iconname);
    classh.res_name = "xwrits";
    classh.res_class = "XWrits";
    XSetWMProperties(display, window, &windowname, &iconname,
		     NULL, 0, NULL, NULL, &classh);
    
    xsh = XAllocSizeHints();
    xsh->x = cenx;
    xsh->y = ceny;
    xsh->flags = USPosition | PMinSize | PMaxSize;
    xsh->min_width = xsh->max_width = WindowWidth;
    xsh->min_height = xsh->max_height = WindowHeight;
    XSetWMNormalHints(display, window, xsh);
  }
  
  SETUPPIX(writs1, PixBreak1);
  SETUPPIX(writs2, PixBreak2);
  SETUPPIX(writsres, PixResting);
  SETUPPIX(writsok, PixOK);
  
  gc = XCreateGC(display, window, 0, 0);
  
  XSelectInput(display, window, ButtonPressMask | ExposureMask);
  
  while (1) {
    
    struct timeval timeout;
    time_t breakclock;
    int picture;
    int deltaseconds; /* use this to get rid of unsigned worries */
    fd_set xfds;
    
    /* Wait for a while, until the person is supposed to take a wrist break. */
    timeout.tv_sec = deltaseconds = 60 * typetime;
    breakclock = time(0) + timeout.tv_sec;
    while (deltaseconds > 0) {
      timeout.tv_usec = 0;
      select(0, 0, 0, 0, &timeout);
      timeout.tv_sec = deltaseconds = breakclock - time(0);
    }
    
    /* Map the window; wait for the Expose before continuing */
    XMapRaised(display, window);
    XFlush(display);
    while (XPending(display)) {
      XNextEvent(display, &event);
      if (event.type == Expose) break;
    }
    
    picture = PixBreak1;
    setpicture(picture);
    
    /* Flash the wrist until the person clicks on the window */
    breakclock = time(0) + flashseconds;
    timeout.tv_sec = flashseconds;
    timeout.tv_usec = 0;
    while (1) {
      FD_ZERO(&xfds);
      FD_SET(xsocket, &xfds);
      select(xsocket + 1, &xfds, 0, 0, &timeout);
      if (FD_ISSET(xsocket, &xfds))
	while (XPending(display)) {
	  XNextEvent(display, &event);
	  if (event.type == ButtonPress) goto takeabreak;
	}
      timeout.tv_sec = deltaseconds = breakclock - time(0);
      timeout.tv_usec = 0;
      if (deltaseconds <= 0) {
	timeout.tv_sec = flashseconds;
	breakclock = time(0) + flashseconds;
	picture ^= PixBreak1 ^ PixBreak2;
	setpicture(picture);
      }
    }
    
   takeabreak:
    
    /* Take a wrist break */
    setpicture(PixResting);
    while (XPending(display))
      XNextEvent(display, &event);
    timeout.tv_sec = deltaseconds = 60 * breaktime;
    breakclock = time(0) + timeout.tv_sec;
    while (deltaseconds > 0) {
      timeout.tv_usec = 0;
      select(0, 0, 0, 0, &timeout);
      timeout.tv_sec = deltaseconds = breakclock - time(0);
    }
    
    /* Wait until they click on us -- */
    setpicture(PixOK);
    while (1) {
      XNextEvent(display, &event);
      if (event.type == ButtonPress) break;
    }
    
    /* then disappear and wait for the MapNotify event before continuing */
    XUnmapWindow(display, window);
    XFlush(display);
    while (XPending(display))
      XNextEvent(display, &event);
    
  }
  
}
